#!/bin/bash

##############

CRIBL_BIN_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
CRIBL_HOME="$( cd "$( dirname "${CRIBL_BIN_DIR}" )" >/dev/null 2>&1 && pwd )"

APP_NAME="Cribl"
KWARGS="server"
PID_DIR="$CRIBL_HOME/pid"
PID_FILE="$PID_DIR/cribl.pid"
LOG_DIR="$CRIBL_HOME/log"
# cribl logs into a file and stdout
LOG_FILE="/dev/null"

###############


USAGE="Usage: $0 {start|stop|restart|reload|status}"

get_host_port_message() {
    cribl_bootup_log_message="API Server is available at"
    output_message="$(grep $cribl_bootup_log_message $LOG_DIR/cribl.log 2> /dev/null|tail -1 2> /dev/null|sed -n 's/.*"message":"//p' 2> /dev/null|sed -n 's/"}//p' 2> /dev/null)"
    if [ -z "$output_message" ]
    then
        output_message="$(grep $cribl_bootup_log_message $LOG_DIR/cribl.log 2> /dev/null|tail -1 2> /dev/null| awk -F '"message":"' '{print $2}' 2> /dev/null| awk -F '"}' '{print $1}' 2> /dev/null)"
    fi
    echo $output_message
}

pid_file_exists() {
    [ -f "$PID_FILE" ]
}

get_pid() {
    echo "$(cat "$PID_FILE")"
}

is_running() {
    PID="$(get_pid)"
    kill -0 $PID 2>/dev/null
    [ $? -eq 0 ]
}

start_it() {
    mkdir -p "$PID_DIR"
    mkdir -p "$LOG_DIR"

    cd $CRIBL_HOME
    if [ $? -ne 0 ]; then
        echo "Could change dir into $CRIBL_HOME"
        exit $?
    fi
    echo -n "Starting $APP_NAME "
    ${CRIBL_HOME}/bin/cribl ${KWARGS}  >> ${LOG_FILE} 2>&1 &
    pid=$!
    echo $pid > $PID_FILE
    for i in {1..10}
    do
        kill -0 $pid 2>/dev/null
        if [ $? -ne 0 ]
        then
            echo "";
            remove_pid_file
            echo "Failed to start $APP_NAME, for more info please check: $LOG_DIR/cribl.log"
            return;
        fi
        echo -n "."
        sleep 1;
    done
    echo ""
    echo "$APP_NAME started with pid $(get_pid)."
    echo "$(get_host_port_message)"
}

stop_process() {
    PID=$(get_pid)
    echo -n "Stopping Cribl, process $PID "
    kill $PID
    while [[ ${?} == 0 ]]
    do
      sleep 1
      echo -n '.'
      kill -0 $PID 2>/dev/null 
    done
    echo
}

remove_pid_file() {
    echo "Removing pid file: $PID_FILE"
    rm -f "$PID_FILE"
}

start_app() {
    if pid_file_exists
    then
        if is_running
        then
            PID=$(get_pid)
            echo "$APP_NAME already running with pid $PID"
            return 1
        else
            echo "$APP_NAME stopped, but pid file exists: $PID_FILE, forcing start anyways"
            remove_pid_file
            start_it
            return 0
        fi
    else
        start_it
        return 0
    fi
    return 1
}

stop_app() {
    if pid_file_exists
    then
        if is_running
        then
            stop_process
        else
            echo "$APP_NAME already stopped but pid file exists, removing stale pid file"
        fi
        remove_pid_file
        echo "$APP_NAME stopped"
    else
        echo "$APP_NAME already stopped, pid file does not exist"
        return 1
    fi
    return 0
}

status_app() {
    if pid_file_exists
    then
        if is_running
        then
            PID=$(get_pid)
            echo "$APP_NAME running with pid $PID."
            echo "$(get_host_port_message)"
        else
            echo "$APP_NAME stopped, but pid file exists"
        fi
    else
        echo "$APP_NAME is stopped"
    fi
    return 0
}

reload_app() {
    if is_running; then
        touch "$CRIBL_HOME/local/cribl/reload"
        echo "Reload request submitted to $APP_NAME"
        return 0
    else
        echo "$APP_NAME is stopped"
        return 1
    fi
}

case "$1" in
    start)
        start_app
    ;;

    stop)
        stop_app
    ;;

    restart)
        stop_app
        start_app
    ;;

    reload)
        reload_app
    ;;

    status)
        status_app
    ;;

    *)
        echo $USAGE
        exit 1
    ;;
esac

exit $?
